@@ -7,7 +7,6 @@
  * of patent rights can be found in the PATENTS file in the same directory.
  *
  * @providesModule ScrollView
- * @flow
  */
 'use strict';
 
@@ -25,6 +24,7 @@ const {
   requireNativeComponent,
 } = ReactNative;
 
+const createReactClass = require('create-react-class');
 const PropTypes = require('prop-types');
 const ColorPropType = require('react-native/Libraries/StyleSheet/ColorPropType');
 const ScrollResponder = require('react-native/Libraries/Components/ScrollResponder');
@@ -36,6 +36,9 @@ const ViewStylePropTypes = require('react-native/Libraries/Components/View/ViewS
 const dismissKeyboard = require('react-native/Libraries/Utilities/dismissKeyboard');
 const invariant = require('fbjs/lib/invariant');
 const processDecelerationRate = require('react-native/Libraries/Components/ScrollView/processDecelerationRate');
+const warning = require('fbjs/lib/warning');
+
+import type {NativeMethodsMixinType} from 'ReactNativeTypes';
 
 /**
  * Forked from https://github.com/facebook/react-native/blob/0.45-stable/Libraries/Components/ScrollView/ScrollView.js
@@ -75,7 +78,8 @@ const processDecelerationRate = require('react-native/Libraries/Components/Scrol
  * supports out of the box.
  */
 // $FlowFixMe(>=0.41.0)
-const ScrollView = React.createClass({
+const ScrollView = createReactClass({
+  displayName: 'ScrollView',
   propTypes: {
     ...ViewPropTypes,
     /**
@@ -138,16 +142,18 @@ const ScrollView = React.createClass({
      * These styles will be applied to the scroll view content container which
      * wraps all of the child views. Example:
      *
-     *   return (
-     *     <ScrollView contentContainerStyle={styles.contentContainer}>
-     *     </ScrollView>
-     *   );
-     *   ...
-     *   const styles = StyleSheet.create({
-     *     contentContainer: {
-     *       paddingVertical: 20
-     *     }
-     *   });
+     * ```
+     * return (
+     *   <ScrollView contentContainerStyle={styles.contentContainer}>
+     *   </ScrollView>
+     * );
+     * ...
+     * const styles = StyleSheet.create({
+     *   contentContainer: {
+     *     paddingVertical: 20
+     *   }
+     * });
+     * ```
      */
     contentContainerStyle: StyleSheetPropType(ViewStylePropTypes),
     /**
@@ -156,8 +162,10 @@ const ScrollView = React.createClass({
      * shortcuts `"normal"` and `"fast"` which match the underlying iOS settings
      * for `UIScrollViewDecelerationRateNormal` and
      * `UIScrollViewDecelerationRateFast` respectively.
-     *   - normal: 0.998 (the default)
-     *   - fast: 0.99
+     *
+     *   - `'normal'`: 0.998 (the default)
+     *   - `'fast'`: 0.99
+     *
      * @platform ios
      */
     decelerationRate: PropTypes.oneOfType([
@@ -171,9 +179,11 @@ const ScrollView = React.createClass({
     horizontal: PropTypes.bool,
     /**
      * The style of the scroll indicators.
-     *   - `default` (the default), same as `black`.
-     *   - `black`, scroll indicator is black. This style is good against a light background.
-     *   - `white`, scroll indicator is white. This style is good against a dark background.
+     *
+     *   - `'default'` (the default), same as `black`.
+     *   - `'black'`, scroll indicator is black. This style is good against a light background.
+     *   - `'white'`, scroll indicator is white. This style is good against a dark background.
+     *
      * @platform ios
      */
     indicatorStyle: PropTypes.oneOf([
@@ -195,9 +205,10 @@ const ScrollView = React.createClass({
     canCancelContentTouches: PropTypes.bool,
     /**
      * Determines whether the keyboard gets dismissed in response to a drag.
-     *   - 'none' (the default), drags do not dismiss the keyboard.
-     *   - 'on-drag', the keyboard is dismissed when a drag begins.
-     *   - 'interactive', the keyboard is dismissed interactively with the drag and moves in
+     *
+     *   - `'none'` (the default), drags do not dismiss the keyboard.
+     *   - `'on-drag'`, the keyboard is dismissed when a drag begins.
+     *   - `'interactive'`, the keyboard is dismissed interactively with the drag and moves in
      *     synchrony with the touch; dragging upwards cancels the dismissal.
      *     On android this is not supported and it will have the same behavior as 'none'.
      */
@@ -209,14 +220,14 @@ const ScrollView = React.createClass({
     /**
      * Determines when the keyboard should stay visible after a tap.
      *
-     *   - 'never' (the default), tapping outside of the focused text input when the keyboard
+     *   - `never` (the default), tapping outside of the focused text input when the keyboard
      *     is up dismisses the keyboard. When this happens, children won't receive the tap.
-     *   - 'always', the keyboard will not dismiss automatically, and the scroll view will not
+     *   - `always`, the keyboard will not dismiss automatically, and the scroll view will not
      *     catch taps, but children of the scroll view can catch taps.
-     *   - 'handled', the keyboard will not dismiss automatically when the tap was handled by
+     *   - `handled`, the keyboard will not dismiss automatically when the tap was handled by
      *     a children, (or captured by an ancestor).
-     *   - false, deprecated, use 'never' instead
-     *   - true, deprecated, use 'always' instead
+     *   - `false`, deprecated, use 'never' instead
+     *   - `true`, deprecated, use 'always' instead
      */
     keyboardShouldPersistTaps: PropTypes.oneOf(['always', 'never', 'handled', false, true]),
     /**
@@ -256,7 +267,7 @@ const ScrollView = React.createClass({
      */
     pagingEnabled: PropTypes.bool,
     /**
-     * When false, the content does not scroll.
+     * When false, the view cannot be scrolled via touch interaction
      * The default value is true.
      */
     scrollEnabled: PropTypes.bool,
@@ -308,17 +319,21 @@ const ScrollView = React.createClass({
     /**
      * When set, causes the scroll view to stop at multiples of the value of
      * `snapToInterval`. This can be used for paginating through children
-     * that have lengths smaller than the scroll view. Used in combination
-     * with `snapToAlignment`.
+     * that have lengths smaller than the scroll view. Typically used in
+     * combination with `snapToAlignment` and `decelerationRate="fast"`.
+     * Overrides less configurable `pagingEnabled` prop.
+     *
      * @platform ios
      */
     snapToInterval: PropTypes.number,
     /**
      * When `snapToInterval` is set, `snapToAlignment` will define the relationship
      * of the snapping to the scroll view.
-     *   - `start` (the default) will align the snap at the left (horizontal) or top (vertical)
-     *   - `center` will align the snap in the center
-     *   - `end` will align the snap at the right (horizontal) or bottom (vertical)
+     *
+     *   - `'start'` (the default) will align the snap at the left (horizontal) or top (vertical)
+     *   - `'center'` will align the snap in the center
+     *   - `'end'` will align the snap at the right (horizontal) or bottom (vertical)
+     *
      * @platform ios
      */
     snapToAlignment: PropTypes.oneOf([
@@ -383,6 +398,15 @@ const ScrollView = React.createClass({
       'always',
       'never',
     ]),
+    /**
+     * When true, ScrollView will emit updateChildFrames data in scroll events,
+     * otherwise will not compute or emit child frame data.  This only exists
+     * to support legacy issues, `onLayout` should be used instead to retrieve
+     * frame data.
+     * The default value is false.
+     * @platform ios
+     */
+    DEPRECATED_sendUpdatedChildFrames: PropTypes.bool
   },
 
   mixins: [ScrollResponder.Mixin],
@@ -490,24 +514,30 @@ const ScrollView = React.createClass({
     this.scrollTo({x, y, animated: false});
   },
 
+  /**
+   * Displays the scroll indicators momentarily.
+   *
+   * @platform ios
+   */
+  flashScrollIndicators: function() {
+    this.getScrollResponder().scrollResponderFlashScrollIndicators();
+  },
+
   _getKeyForIndex: function(index, childArray) {
     const child = childArray[index];
     return child && child.key;
   },
 
   _updateAnimatedNodeAttachment: function() {
+    if (this._scrollAnimatedValueAttachment) {
+      this._scrollAnimatedValueAttachment.detach();
+    }
     if (this.props.stickyHeaderIndices && this.props.stickyHeaderIndices.length > 0) {
-      if (!this._scrollAnimatedValueAttachment) {
-        this._scrollAnimatedValueAttachment = Animated.attachNativeEvent(
-          this._scrollViewRef,
-          'onScroll',
-          [{nativeEvent: {contentOffset: {y: this._scrollAnimatedValue}}}]
-        );
-      }
-    } else {
-      if (this._scrollAnimatedValueAttachment) {
-        this._scrollAnimatedValueAttachment.detach();
-      }
+      this._scrollAnimatedValueAttachment = Animated.attachNativeEvent(
+        this._scrollViewRef,
+        'onScroll',
+        [{nativeEvent: {contentOffset: {y: this._scrollAnimatedValue}}}]
+      );
     }
   },
 
@@ -572,8 +602,8 @@ const ScrollView = React.createClass({
     this._scrollViewRef = ref;
   },
 
-  _innerViewRef: (null: ?View),
-  _setInnerViewRef: function(ref: ?View) {
+  _innerViewRef: (null: ?NativeMethodsMixinType),
+  _setInnerViewRef: function(ref: ?NativeMethodsMixinType) {
     this._innerViewRef = ref;
   },
 
@@ -583,6 +613,10 @@ const ScrollView = React.createClass({
     if (Platform.OS === 'ios') {
       ScrollViewClass = RCTScrollView;
       ScrollContentContainerViewClass = RCTScrollContentView;
+      warning(
+        !this.props.snapToInterval || !this.props.pagingEnabled,
+        'snapToInterval is currently ignored when pagingEnabled is true.'
+      );
     } else if (Platform.OS === 'android') {
       if (this.props.horizontal) {
         ScrollViewClass = AndroidHorizontalScrollView;
@@ -656,7 +690,13 @@ const ScrollView = React.createClass({
         {...contentSizeChangeProps}
         ref={this._setInnerViewRef}
         style={contentContainerStyle}
-        removeClippedSubviews={this.props.removeClippedSubviews}
+        removeClippedSubviews={
+          // Subview clipping causes issues with sticky headers on Android and
+          // would be hard to fix properly in a performant way.
+          Platform.OS === 'android' && hasStickyHeaders ?
+            false :
+            this.props.removeClippedSubviews
+        }
         collapsable={false}>
         {children}
       </ScrollContentContainerViewClass>;
@@ -671,6 +711,10 @@ const ScrollView = React.createClass({
         this.props.alwaysBounceVertical :
         !this.props.horizontal;
 
+
+    const DEPRECATED_sendUpdatedChildFrames =
+      !!this.props.DEPRECATED_sendUpdatedChildFrames;
+
     const baseStyle = this.props.horizontal ? styles.baseHorizontal : styles.baseVertical;
     const props = {
       ...this.props,
@@ -699,6 +743,7 @@ const ScrollView = React.createClass({
       scrollEventThrottle: hasStickyHeaders ? 1 : this.props.scrollEventThrottle,
       sendMomentumEvents: (this.props.onMomentumScrollBegin || this.props.onMomentumScrollEnd) ?
         true : false,
+      DEPRECATED_sendUpdatedChildFrames
     };
 
     const { decelerationRate } = this.props;
@@ -795,6 +840,7 @@ if (Platform.OS === 'android') {
     (ScrollView: ReactClass<any>),
     nativeOnlyProps,
   );
+  // $FlowFixMe (bvaughn) Update ComponentInterface in ViewPropTypes to include a string type (for Fiber host components) in a follow-up.
   RCTScrollContentView = requireNativeComponent('RCTScrollContentView', View);
 }
 
